{----------------------------------------------------}
{ Andreis LCARS-Components 1.1 for Delphi (2, 3), 4 }
{----------------------------------------------------}
{ Copyright (c) 1999 Andrei Scheibner                }
{ http://www.htwm.de/~ascheibn                       }
{ ascheibn@htwm.de                                   }
{----------------------------------------------------}
{These Components are Freeware. You can distribute them}
{altogether, you can improve them , but I would appreciate}
{it, if my name appears somewhere in your improvements.}

unit LCARSGauge;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

const MarkerThickness = 4;
      GridStepWidth = 5;
      SpaceBetweenGrids = 2;
      FGridOneMin = 0;

type
  TGaugeStyle = (gsNormal, gsGridded);
  TGaugeDirection = (gdUp, gdRight, gdDown, gdLeft);
  TGaugeGrids = (ggOne, ggTwo, ggThree, ggFour);
  TLCARSGauge = class(TGraphicControl)
  private
   FMinValue: Integer;
   FMaxValue: Integer;
   FCurValue: Integer;
   FStyle: TGaugeStyle;
   FShowText: Boolean;
   FGBounds : Boolean;
   FGBoundsThickness : Integer;
   FGBoundsColor : TColor;
   FGBarColor : TColor;
   FGaugeDirection : TGaugeDirection;
   FBackColor: TColor;
   FGaugeGrids : TGaugeGrids;
   FGridTwoAtPercent : Integer;
   FGridThreeAtPercent : Integer;
   FGridFourAtPercent : Integer;
   FZeroPercent : Integer;
   FHundredPercent : Integer;
   FGridOneColor : TColor;
   FGridTwoColor : TColor;
   FGridThreeColor : TColor;
   FGridFourColor : TColor;
   FGLEveryNPercent : Integer;
   FTextPlace : Integer;
   FBarPlace : Integer;
   FPercentPlace : Integer;
   FProgressPlace : Integer;
   FProgressToShow : Integer;
   FOldToShow : Integer;
   FTextWidth : Integer;
   FTextHeight : Integer;
   FTextPosX : Integer;
   FTextPosY : Integer;
   FGridTwoMin : Integer;
   FGridThreeMin : Integer;
   FGridFourMin : Integer;
   FGridOneMax : Integer;
   FGridTwoMax : Integer;
   FGridThreeMax : Integer;
   FGridFourMax : Integer;
   FOnChange : TNotifyEvent;
   FBack : TBitmap;
   FMouseInControl: Boolean;
   FOnMouseEnter: TNotifyEvent;
   FOnMouseLeave: TNotifyEvent;
   procedure SetGaugeStyle(Value: TGaugeStyle);
   procedure SetShowText(Value: Boolean);
   procedure SetMinValue(Value: Integer);
   procedure SetMaxValue(Value: Integer);
   procedure SetProgress(Value: Integer);
   procedure SetGBounds(Value : Boolean);
   procedure SetGBoundsColor(value : TColor);
   procedure SetGBarColor(value : TColor);
   procedure SetGaugeGrids(value : TGaugeGrids);
   procedure SetGridTwoAtPercent(value : Integer);
   procedure SetGridThreeAtPercent(value : Integer);
   procedure SetGridFourAtPercent(value : Integer);
   procedure SetGridOneColor(Value: TColor);
   procedure SetGridTwoColor(Value: TColor);
   procedure SetGridThreeColor(Value: TColor);
   procedure SetGridFourColor(Value: TColor);
   procedure SetGaugeDirection(Value : TGaugeDirection);
   procedure SetGLEveryNPercent(value : Integer);
   function CalculateNewProgress : boolean;
   procedure DrawGridBounds;
   procedure DrawGridOne;
   procedure DrawGridTwo;
   procedure DrawGridThree;
   procedure DrawGridFour;
   procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
   procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
   procedure Paint; override;
   procedure Change ; dynamic;
   procedure MouseEnter; dynamic;
   procedure MouseLeave; dynamic;
  public
   property MouseInControl: Boolean read FMouseInControl;
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure AddProgress(Value: Integer);
  published
   property Align;
   property Anchors;
   property Constraints;
   property Enabled;
   property Font;
   property GaugeDirection : TGaugeDirection read FGaugeDirection
    write SetGaugeDirection;
   property GaugeGrids : TGaugeGrids read FGaugeGrids write SetGaugeGrids;
   property GridTwoAtPercent : Integer read FGridTwoAtPercent
    write SetGridTwoAtPercent;
   property GridThreeAtPercent : Integer read FGridThreeAtPercent
    write SetGridThreeAtPercent;
   property GridFourAtPercent : Integer read FGridFourAtPercent
    write SetGridFourAtPercent;
   property GridOneColor : TColor read FGridOneColor write SetGridOneColor;
   property GridTwoColor : TColor read FGridTwoColor write SetGridTwoColor;
   property GridThreeColor : TColor read FGridThreeColor write SetGridThreeColor;
   property GridFourColor : TColor read FGridFourColor write SetGridFourColor;
   property BarColor : TColor read FGBarColor write SetGBarColor;
   property BoundsColor : TColor read FGBoundsColor write SetGBoundsColor;
   property GaugeBounds : Boolean read FGBounds write SetGBounds default false;
   property GaugeStyle: TGaugeStyle read FStyle write SetGaugeStyle default gsNormal;
   property GLEveryNPercent : Integer read FGLEveryNPercent
    write SetGLEveryNPercent;
   property MinValue: Integer read FMinValue write SetMinValue default 0;
   property MaxValue: Integer read FMaxValue write SetMaxValue default 100;
   property ParentColor;
   property ParentFont default true;
   property ParentShowHint;
   property Progress: Integer read FCurValue write SetProgress default 0;
   property ShowHint;
   property ShowText: Boolean read FShowText write SetShowText default True;
   property Visible;
   property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
   property OnMouseLeave: TNotifyEvent read FOnMouseLeave write FOnMouseLeave;
   property OnChange: TNotifyEvent read FOnChange write FOnChange;
   property OnClick;
   property OnMouseDown;
   property OnMouseMove;
   property OnMouseUp;
   property OnDragDrop;
   property OnDragOver;
   property OnEndDrag;
  end;

procedure Register;

implementation

constructor TLCARSGauge.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FProgressPlace := 1;
 Width := 50;
 Height := 200;
 FMinValue := 0;
 FMaxValue := 100;
 Progress := 0;
 FStyle := gsNormal;
 FShowText := True;
 FGLEveryNPercent := 5;
 FGBoundsColor := clyellow;
 FGridOneColor := clyellow;
 FGridTwoColor := clgreen;
 FGridTwoAtPercent := 25;
 FGridThreeColor := clblue;
 FGridThreeAtPercent := 50;
 FGridFourColor := clnavy;
 FGridFourAtPercent := 75;
 FGBarColor := clred;
 Canvas.Brush.Style := bsClear;
 ControlStyle := ControlStyle + [csOpaque];
 FBack := TBitmap.Create;
end;

destructor TLCARSGauge.Destroy;
begin
 FBack.Free;
 inherited Destroy;
end;

procedure TLCARSGauge.DrawGridBounds;
begin
 if FGaugeGrids = ggOne then
  begin
   FGridOneMax := FBarPlace;
   DrawGridOne;
  end;
 if FGaugeGrids = ggTwo then
  begin
   FGridTwoMax := FBarPlace;
   FGridTwoMin := Round((FBarPlace / 100) * FGridTwoAtPercent);
   FGridOneMax := FGridTwoMin - SpaceBetweenGrids;
   DrawGridOne;
   DrawGridTwo;
  end;
 if FGaugeGrids = ggThree then
  begin
   FGridThreeMax := FBarPlace;
   FGridThreeMin := Round((FBarPlace / 100) * FGridThreeAtPercent);
   FGridTwoMax := FGridThreeMin - SpaceBetweenGrids;
   FGridTwoMin := Round((FBarPlace / 100) * FGridTwoAtPercent);
   FGridOneMax := FGridTwoMin - SpaceBetweenGrids;
   DrawGridOne;
   DrawGridTwo;
   DrawGridThree;
  end;
 if FGaugeGrids = ggFour then
  begin
   FGridFourMax := FBarPlace;
   FGridFourMin := Round((FBarPlace / 100) * FGridFourAtPercent);
   FGridThreeMax := FGridFourMin - SpaceBetweenGrids;
   FGridThreeMin := Round((FBarPlace / 100) * FGridThreeAtPercent);
   FGridTwoMax := FGridThreeMin - SpaceBetweenGrids;
   FGridTwoMin := Round((FBarPlace / 100) * FGridTwoAtPercent);
   FGridOneMax := FGridTwoMin - SpaceBetweenGrids;
   DrawGridOne;
   DrawGridTwo;
   DrawGridThree;
   DrawGridFour;
  end;
end;

procedure TLCARSGauge.DrawGridOne;
var counter : integer;
    BoundRect : TRect;
    pmax, LineDistance, NextGridLine : real;
begin
 FBack.Canvas.Brush.Color := GridOneColor;
 FBack.Canvas.Pen.Color := GridOneColor;
 if FGaugeGrids = ggOne then
  pmax := 100
 else
  pmax := FGridTwoAtPercent;
 LineDistance :=  (FGridOneMax - FGridOneMin) / (pmax / FGLEveryNPercent);
 NextGridLine := FGridOneMin;
 if FGaugeDirection = gdRight then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FGridOneMin + counter, counter,
                       FGridOneMax - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridOneMin;
   while counter < FGridOneMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridOneMax then
      begin
       FBack.Canvas.MoveTo(counter, FGBoundsThickness);
       FBack.Canvas.LineTo(counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdLeft then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FBack.Width - FGridOneMax + counter, counter,
                       FBack.Width - FGridOneMin - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridOneMin;
   while counter < FGridOneMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridOneMax then
      begin
       FBack.Canvas.MoveTo(FBack.Width - counter, FGBoundsThickness);
       FBack.Canvas.LineTo(FBack.Width - counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdUp then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FBack.Height - FGridOneMax + counter,
                       FBack.Width - counter, FBack.Height - FGridOneMin - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridOneMin;
   while counter < FGridOneMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridOneMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, FBack.Height - counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, FBack.Height - counter);
      end;
    end;
  end;
 if FGaugeDirection = gdDown then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FGridOneMin + counter,
                       FBack.Width - counter, FGridOneMax - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridOneMin;
   while counter < FGridOneMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridOneMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, counter);
      end;
    end;
  end;
end;

procedure TLCARSGauge.DrawGridTwo;
var counter : integer;
    BoundRect : TRect;
    pmax, LineDistance, NextGridLine : real;
begin
 pmax := 1;
 FBack.Canvas.Brush.Color := GridTwoColor;
 FBack.Canvas.Pen.Color := GridTwoColor;
 if FGaugeGrids = ggTwo then
  pmax := 100
 else if (FGaugeGrids = ggThree) or (FGaugeGrids = ggFour) then
  pmax := FGridThreeAtPercent;
 LineDistance :=  (FGridTwoMax - FGridTwoMin) / ((pmax - FGridTwoAtPercent) / FGLEveryNPercent);
 NextGridLine := FGridTwoMin;
 if FGaugeDirection = gdRight then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FGridTwoMin + counter, counter,
                       FGridTwoMax - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridTwoMin;
   while counter < FGridTwoMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridTwoMax then
      begin
       FBack.Canvas.MoveTo(counter, FGBoundsThickness);
       FBack.Canvas.LineTo(counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdLeft then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FBack.Width - FGridTwoMax + counter, counter,
                       FBack.Width - FGridTwoMin - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridTwoMin;
   while counter < FGridTwoMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridTwoMax then
      begin
       FBack.Canvas.MoveTo(FBack.Width - counter, FGBoundsThickness);
       FBack.Canvas.LineTo(FBack.Width - counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdUp then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FBack.Height - FGridTwoMax + counter,
                       FBack.Width - counter, FBack.Height - FGridTwoMin - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridTwoMin;
   while counter < FGridTwoMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridTwoMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, FBack.Height - counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, FBack.Height - counter);
      end;
    end;
  end;
 if FGaugeDirection = gdDown then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FGridTwoMin + counter,
                       FBack.Width - counter, FGridTwoMax - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridTwoMin;
   while counter < FGridTwoMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridTwoMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, counter);
      end;
    end;
  end;
end;

procedure TLCARSGauge.DrawGridThree;
var counter : integer;
    BoundRect : TRect;
    pmax, LineDistance, NextGridLine : real;
begin
 pmax := 1;
 FBack.Canvas.Brush.Color := GridThreeColor;
 FBack.Canvas.Pen.Color := GridThreeColor;
 if FGaugeGrids = ggThree then
  pmax := 100
 else if FGaugeGrids = ggFour then
  pmax := FGridFourAtPercent;
 LineDistance :=  (FGridThreeMax - FGridThreeMin) / ((pmax - FGridThreeAtPercent) / FGLEveryNPercent);
 NextGridLine := FGridThreeMin;
 if FGaugeDirection = gdRight then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FGridThreeMin + counter, counter,
                       FGridThreeMax - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridThreeMin;
   while counter < FGridThreeMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridThreeMax then
      begin
       FBack.Canvas.MoveTo(counter, FGBoundsThickness);
       FBack.Canvas.LineTo(counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdLeft then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FBack.Width - FGridThreeMax + counter, counter,
                       FBack.Width - FGridThreeMin - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridThreeMin;
   while counter < FGridThreeMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridThreeMax then
      begin
       FBack.Canvas.MoveTo(FBack.Width - counter, FGBoundsThickness);
       FBack.Canvas.LineTo(FBack.Width - counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdUp then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FBack.Height - FGridThreeMax + counter,
                       FBack.Width - counter, FBack.Height - FGridThreeMin - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridThreeMin;
   while counter < FGridThreeMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridThreeMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, FBack.Height - counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, FBack.Height - counter);
      end;
    end;
  end;
 if FGaugeDirection = gdDown then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FGridThreeMin + counter,
                       FBack.Width - counter, FGridThreeMax - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridThreeMin;
   while counter < FGridThreeMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridThreeMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, counter);
      end;
    end;
  end;
end;

procedure TLCARSGauge.DrawGridFour;
var counter : integer;
    BoundRect : TRect;
    pmax, LineDistance, NextGridLine : real;
begin
 FBack.Canvas.Brush.Color := GridFourColor;
 FBack.Canvas.Pen.Color := GridFourColor;
 pmax := 100;
 LineDistance :=  (FGridFourMax - FGridFourMin) / ((pmax - FGridFourAtPercent) / FGLEveryNPercent);
 NextGridLine := FGridFourMin;
 if FGaugeDirection = gdRight then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FGridFourMin + counter, counter,
                       FGridFourMax - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridFourMin;
   while counter < FGridFourMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridFourMax then
      begin
       FBack.Canvas.MoveTo(counter, FGBoundsThickness);
       FBack.Canvas.LineTo(counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdLeft then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(FBack.Width - FGridFourMax + counter, counter,
                       FBack.Width - FGridFourMin - counter, FBack.Height - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridFourMin;
   while counter < FGridFourMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridFourMax then
      begin
       FBack.Canvas.MoveTo(FBack.Width - counter, FGBoundsThickness);
       FBack.Canvas.LineTo(FBack.Width - counter, FBack.Height - FGBoundsThickness);
      end;
    end;
  end;
 if FGaugeDirection = gdUp then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FBack.Height - FGridFourMax + counter,
                       FBack.Width - counter, FBack.Height - FGridFourMin - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridFourMin;
   while counter < FGridFourMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridFourMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, FBack.Height - counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, FBack.Height - counter);
      end;
    end;
  end;
 if FGaugeDirection = gdDown then
  begin
   for counter := 0 to FGBoundsThickness - 1 do
    begin
     BoundRect := Rect(counter, FGridFourMin + counter,
                       FBack.Width - counter, FGridFourMax - counter);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   counter := FGridFourMin;
   while counter < FGridFourMax do
    begin
     NextGridLine := NextGridLine + LineDistance;
     counter := Round(NextGridLine);
     if counter < FGridFourMax then
      begin
       FBack.Canvas.MoveTo(FGBoundsThickness, counter);
       FBack.Canvas.LineTo(FBack.Width - FGBoundsThickness, counter);
      end;
    end;
  end;
end;

procedure TLCARSGauge.Paint;
var VisibleRect, BackRect, BoundRect : TRect;
begin
 FBack.Canvas.Font.Assign(Font);
 FBack.Width := Width;
 FBack.Height := Height;
 if Parent <> nil then
  with Parent do
   FBackColor := Color;
 FBack.Canvas.Brush.Color := FBackColor;
 FBack.Canvas.Pen.Color := FBackColor;
 FBack.Canvas.Rectangle(0,0, FBack.width, FBack.height);
 if FStyle = gsNormal then
  begin
   if FGBounds then
    FGBoundsThickness := 2
   else
    FGBoundsThickness := 0;
  end
 else if FStyle = gsGridded then
  FGBoundsThickness := 3;
 if FShowText then
  begin
   if (FGaugeDirection = gdLeft) or (FGaugeDirection = gdRight) then
    FTextPlace := FBack.Canvas.TextWidth('88888') + 4
   else if (FGaugeDirection = gdUp) or (FGaugeDirection = gdDown) then
    FTextPlace := FBack.Canvas.TextHeight('88888') + 4;
   FTextWidth := FBack.Canvas.TextWidth(InttoStr(Progress));
   FTextHeight := FBack.Canvas.TextHeight(InttoStr(Progress));
  end
 else
  FTextPlace := 0;
 if (FGaugeDirection = gdLeft) or (FGaugeDirection = gdRight) then
  FBarPlace := FBack.Width - FTextPlace
 else if (FGaugeDirection = gdUp) or (FGaugeDirection = gdDown) then
  FBarPlace := FBack.Height - FTextPlace;
 FProgressPlace := MaxValue - MinValue;
 FZeroPercent := FGBoundsThickness + MarkerThickness;
 FHundredPercent := FBarPlace - FGBoundsThickness;
 FPercentPlace := FHundredPercent - FZeroPercent;
 if FGaugeDirection = gdRight then
  begin
   CalculateNewProgress;
   FBack.Canvas.Brush.Color := FGBarColor;
   FBack.Canvas.Pen.Color := FGBarColor;
   FBack.Canvas.Rectangle(FZeroPercent - MarkerThickness, FGBoundsThickness,
                          FZeroPercent + FProgressToShow, FBack.Height - FGBoundsThickness);
   FBack.Canvas.Brush.Color := clWhite;
   FBack.Canvas.Pen.Color := clWhite;
   FBack.Canvas.Rectangle(FZeroPercent + FProgressToShow - MarkerThickness, FGBoundsThickness,
                          FZeroPercent + FProgressToShow, FBack.Height - FGBoundsThickness);
   if (FGBounds) and (FStyle = gsNormal) then
    begin
     FBack.Canvas.Brush.Color := FGBoundsColor;
     BoundRect := Rect(0, 0, FBarPlace, FBack.Height);
     FBack.Canvas.FrameRect(BoundRect);
     BoundRect := Rect(1, 1, FBarPlace-1, FBack.Height-1);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   if FStyle = gsGridded then
    begin
     DrawGridBounds;
    end;
   if FShowText then
    begin
     FBack.Canvas.Brush.Color := FBackColor;
     FBack.Canvas.Pen.Color := FBackColor;
     FTextPosX := (FBack.Width - FTextPlace) + (FTextPlace div 2) - (FTextWidth div 2);
     FTextPosY := (FBack.Height div 2) - (FTextHeight div 2);
     FBack.Canvas.TextOut(FTextPosX, FTextPosY, InttoStr(Progress));
    end;
  end
 else if FGaugeDirection = gdLeft then
  begin
   CalculateNewProgress;
   FBack.Canvas.Brush.Color := FGBarColor;
   FBack.Canvas.Pen.Color := FGBarColor;
   FBack.Canvas.Rectangle(FBack.Width - FZeroPercent - FProgressToShow, FGBoundsThickness,
                          FBack.Width - FZeroPercent + MarkerThickness, FBack.Height - FGBoundsThickness);
   FBack.Canvas.Brush.Color := clWhite;
   FBack.Canvas.Pen.Color := clWhite;
   FBack.Canvas.Rectangle(FBack.Width - FZeroPercent - FProgressToShow, FGBoundsThickness,
                          FBack.Width - FZeroPercent - FProgressToShow + MarkerThickness, FBack.Height - FGBoundsThickness);
   if (FGBounds) and (FStyle = gsNormal) then
    begin
     FBack.Canvas.Brush.Color := FGBoundsColor;
     BoundRect := Rect(FBack.Width - FBarPlace, 0, FBack.Width, FBack.Height);
     FBack.Canvas.FrameRect(BoundRect);
     BoundRect := Rect(FBack.Width - FBarPlace+1, 1, FBack.Width-1, FBack.Height-1);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   if FStyle = gsGridded then
    begin
     DrawGridBounds;
    end;
   if FShowText then
    begin
     FBack.Canvas.Brush.Color := FBackColor;
     FBack.Canvas.Pen.Color := FBackColor;
     FTextPosX := (FTextPlace div 2) - (FTextWidth div 2);
     FTextPosY := (FBack.Height div 2) - (FTextHeight div 2);
     FBack.Canvas.TextOut(FTextPosX, FTextPosY, InttoStr(Progress));
    end;
  end
 else if FGaugeDirection = gdUp then
  begin
   CalculateNewProgress;
   FBack.Canvas.Brush.Color := FGBarColor;
   FBack.Canvas.Pen.Color := FGBarColor;
   FBack.Canvas.Rectangle(FGBoundsThickness, FBack.Height - FZeroPercent - FProgressToShow,
                          FBack.Width - FGBoundsThickness, FBack.Height - FZeroPercent + MarkerThickness);
   FBack.Canvas.Brush.Color := clWhite;
   FBack.Canvas.Pen.Color := clWhite;
   FBack.Canvas.Rectangle(FGBoundsThickness, FBack.Height - FZeroPercent - FProgressToShow,
                          FBack.Width - FGBoundsThickness, FBack.Height - FZeroPercent - FProgressToShow + MarkerThickness);
   if (FGBounds) and (FStyle = gsNormal) then
    begin
     FBack.Canvas.Brush.Color := FGBoundsColor;
     BoundRect := Rect(0, FBack.Height - FBarPlace, FBack.Width, FBack.Height);
     FBack.Canvas.FrameRect(BoundRect);
     BoundRect := Rect(1, FBack.Height - FBarPlace+1, FBack.Width-1, FBack.Height-1);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   if FStyle = gsGridded then
    begin
     DrawGridBounds;
    end;
   if FShowText then
    begin
     FBack.Canvas.Brush.Color := FBackColor;
     FBack.Canvas.Pen.Color := FBackColor;
     FTextPosX := (FBack.Width div 2) - (FTextWidth div 2);
     FTextPosY := (FTextPlace div 2) - (FTextHeight div 2);
     FBack.Canvas.TextOut(FTextPosX, FTextPosY, InttoStr(Progress));
    end;
  end
 else if FGaugeDirection = gdDown then
  begin
   CalculateNewProgress;
   FBack.Canvas.Brush.Color := FGBarColor;
   FBack.Canvas.Pen.Color := FGBarColor;
   FBack.Canvas.Rectangle(FGBoundsThickness, FZeroPercent - MarkerThickness,
                          FBack.Width - FGBoundsThickness, FZeroPercent + FProgressToShow);
   FBack.Canvas.Brush.Color := clWhite;
   FBack.Canvas.Pen.Color := clWhite;
   FBack.Canvas.Rectangle(FGBoundsThickness, FZeroPercent + FProgressToShow - MarkerThickness,
                          FBack.Width - FGBoundsThickness, FZeroPercent + FProgressToShow);
   if (FGBounds) and (FStyle = gsNormal) then
    begin
     FBack.Canvas.Brush.Color := FGBoundsColor;
     BoundRect := Rect(0, 0, FBack.Width, FBarPlace);
     FBack.Canvas.FrameRect(BoundRect);
     BoundRect := Rect(1, 1, FBack.Width-1, FBarPlace-1);
     FBack.Canvas.FrameRect(BoundRect);
    end;
   if FStyle = gsGridded then
    begin
     DrawGridBounds;
    end;
   if FShowText then
    begin
     FBack.Canvas.Brush.Color := FBackColor;
     FBack.Canvas.Pen.Color := FBackColor;
     FTextPosX := (FBack.Width div 2) - (FTextWidth div 2);
     FTextPosY := (FBack.Height - FTextPlace) + (FTextPlace div 2) - (FTextHeight div 2);
     FBack.Canvas.TextOut(FTextPosX, FTextPosY, InttoStr(Progress));
    end;
  end;
 VisibleRect := Rect(0, 0, Width, Height);
 BackRect := Rect(0, 0, FBack.Width, Fback.Height);
 FBack.Transparent := true;
 FBack.TransparentColor := FBackColor;
 Canvas.CopyRect(VisibleRect, FBack.Canvas, BackRect);
end;

function TLCARSGauge.CalculateNewProgress : boolean;
var Factor : real;
begin
 FOldToShow := FProgressToShow;
 Factor := FPercentPlace / FProgressPlace;
 FProgressToShow := Round(Factor * (FCurValue - FMinValue));
 if FProgressToShow = FOldToShow then
  CalculateNewProgress := false
 else
  CalculateNewProgress := true;
end;

procedure TLCARSGauge.SetGaugeStyle(Value: TGaugeStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.Change;
begin
 if Assigned(FOnChange) then FOnChange(Self);
end;

procedure TLCARSGauge.SetGaugeGrids(Value: TGaugeGrids);
begin
  if Value <> FGaugeGrids then
  begin
    FGaugeGrids := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGridTwoAtPercent(Value: Integer);
begin
 FGridTwoAtPercent := (Value div GridStepWidth) * GridStepWidth;
 Repaint;
end;

procedure TLCARSGauge.SetGridThreeAtPercent(Value: Integer);
begin
 FGridThreeAtPercent := (Value div GridStepWidth) * GridStepWidth;
 Repaint;
end;

procedure TLCARSGauge.SetGridFourAtPercent(Value: Integer);
begin
 FGridFourAtPercent := (Value div GridStepWidth) * GridStepWidth;
 Repaint;
end;

procedure TLCARSGauge.SetShowText(Value: Boolean);
begin
  if Value <> FShowText then
  begin
    FShowText := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGBounds(Value: Boolean);
begin
  if Value <> FGBounds then
  begin
    FGBounds := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGBoundsColor(Value: TColor);
begin
  if Value <> FGBoundsColor then
  begin
    FGBoundsColor := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGBarColor(Value: TColor);
begin
  if Value <> FGBarColor then
  begin
    FGBarColor := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGridOneColor(Value: TColor);
begin
  if Value <> FGridOneColor then
  begin
    FGridOneColor := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGridTwoColor(Value: TColor);
begin
  if Value <> FGridTwoColor then
  begin
    FGridTwoColor := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGridThreeColor(Value: TColor);
begin
  if Value <> FGridThreeColor then
  begin
    FGridThreeColor := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGridFourColor(Value: TColor);
begin
  if Value <> FGridFourColor then
  begin
    FGridFourColor := Value;
    Repaint;
  end;
end;

procedure TLCARSGauge.SetGaugeDirection(Value: TGaugeDirection);
begin
 if Value <> FGaugeDirection then
  begin
   FGaugeDirection := Value;
   Repaint;
  end;
end;

procedure TLCARSGauge.SetGLEveryNPercent(Value : Integer);
begin
 if Value < 10 then
  FGLEveryNPercent := 5
 else
  FGLEveryNPercent := 10;
 Repaint;
end;

procedure TLCARSGauge.SetMinValue(Value: Integer);
begin
  if Value <> FMinValue then
  begin
    if Value > FMaxValue then
     FMinValue := FCurValue
    else
     FMinValue := Value;
    if FCurValue < Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TLCARSGauge.SetMaxValue(Value: Integer);
begin
  if Value <> FMaxValue then
  begin
    if Value < FMinValue then
     FMaxValue := FCurValue
    else
     FMaxValue := Value;
    if FCurValue > Value then FCurValue := Value;
    Refresh;
  end;
end;

procedure TLCARSGauge.SetProgress(Value: Integer);
begin
 if Value < FMinValue then
  Value := FMinValue
 else if Value > FMaxValue then
  Value := FMaxValue;
 if FCurValue <> Value then
  begin
   FCurValue := Value;
   Change;
  end;
 Repaint;
end;

procedure TLCARSGauge.AddProgress(Value: Integer);
begin
  Progress := FCurValue + Value;
end;

procedure TLCARSGauge.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if not FMouseInControl and Enabled {and IsForegroundTask } then begin
    FMouseInControl := True;
    MouseEnter;
  end;
end;

procedure TLCARSGauge.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if FMouseInControl and Enabled then begin
    FMouseInControl := False;
    MouseLeave;
  end;
end;

procedure TLCARSGauge.MouseEnter;
begin
  if Assigned(FOnMouseEnter) then FOnMouseEnter(Self);
end;

procedure TLCARSGauge.MouseLeave;
begin
  if Assigned(FOnMouseLeave) then FOnMouseLeave(Self);
end;

procedure Register;
begin
  RegisterComponents('LCARS-Comps', [TLCARSGauge]);
end;

end.
